@@ -888,6 +888,11 @@ apr_status_t modsecurity_request_body_clear(modsec_rec *msr, char **error_msg) {
                 const char *put_filename = NULL;
                 const char *put_basename = NULL;
 
+                if (strcmp(msr->txcfg->upload_dir, msr->txcfg->tmp_dir) == 0) {
+                    msr_log(msr, 4, "Not moving file to identical location.");
+                    goto nullify;
+                }
+
                 /* Construct the new filename. */
                 put_basename = file_basename(msr->msc_reqbody_mp, msr->msc_reqbody_filename);
                 if (put_basename == NULL) {
@@ -933,6 +938,8 @@ apr_status_t modsecurity_request_body_clear(modsec_rec *msr, char **error_msg) {
                     msr->msc_reqbody_filename);
             }
 
+nullify:
+
             msr->msc_reqbody_filename = NULL;
         }
     }
