@@ -101,6 +101,7 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
     int expired = 0;
     int i;
 
+
     if (msr->txcfg->data_dir == NULL) {
         msr_log(msr, 1, "collection_retrieve_ex: Unable to retrieve collection (name \"%s\", key \"%s\"). Use "
             "SecDataDir to define data directory first.", log_escape(msr->mp, col_name),
@@ -119,10 +120,21 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
     key.dsize = col_key_len + 1;
 
     if (existing_dbm == NULL) {
+#ifdef GLOBAL_COLLECTION_LOCK
+        rc = apr_global_mutex_lock(msr->modsecurity->dbm_lock);
+        if (rc != APR_SUCCESS) {
+            msr_log(msr, 1, "collection_retrieve_ex: Failed to lock proc mutex: %s",
+                    get_apr_error(msr->mp, rc));
+            goto cleanup;
+        }
+#endif
         rc = apr_sdbm_open(&dbm, dbm_filename, APR_READ | APR_SHARELOCK,
             CREATEMODE, msr->mp);
         if (rc != APR_SUCCESS) {
             dbm = NULL;
+#ifdef GLOBAL_COLLECTION_LOCK
+            apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
             goto cleanup;
         }
     }
@@ -156,6 +168,9 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
     /* Close after "value" used from fetch or memory may be overwritten. */
     if (existing_dbm == NULL) {
         apr_sdbm_close(dbm);
+#ifdef GLOBAL_COLLECTION_LOCK
+        apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
         dbm = NULL;
     }
 
@@ -202,12 +217,23 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
      */
     if (apr_table_get(col, "KEY") == NULL) {
         if (existing_dbm == NULL) {
+#ifdef GLOBAL_COLLECTION_LOCK
+            rc = apr_global_mutex_lock(msr->modsecurity->dbm_lock);
+            if (rc != APR_SUCCESS) {
+                msr_log(msr, 1, "collection_retrieve_ex: Failed to lock proc mutex: %s",
+                        get_apr_error(msr->mp, rc));
+                goto cleanup;
+            }
+#endif
             rc = apr_sdbm_open(&dbm, dbm_filename, APR_CREATE | APR_WRITE | APR_SHARELOCK,
                 CREATEMODE, msr->mp);
             if (rc != APR_SUCCESS) {
                 msr_log(msr, 1, "collection_retrieve_ex: Failed to access DBM file \"%s\": %s",
                     log_escape(msr->mp, dbm_filename), get_apr_error(msr->mp, rc));
                 dbm = NULL;
+#ifdef GLOBAL_COLLECTION_LOCK
+                apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
                 goto cleanup;
             }
         }
@@ -217,16 +243,22 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
 
         rc = apr_sdbm_delete(dbm, key);
         if (rc != APR_SUCCESS) {
-            msr_log(msr, 1, "collection_retrieve_ex: Failed deleting collection (name \"%s\", "
-                "key \"%s\"): %s", log_escape(msr->mp, col_name),
-                log_escape_ex(msr->mp, col_key, col_key_len), get_apr_error(msr->mp, rc));
-            msr->msc_sdbm_delete_error = 1;
+#ifdef LOG_NO_COLL_DELET_PB
+		if (msr->txcfg->debuglog_level >= 9)
+#endif
+		msr_log(msr, 1, "collection_retrieve_ex: Failed deleting collection (name \"%s\", "
+			"key \"%s\"): %s", log_escape(msr->mp, col_name),
+			log_escape_ex(msr->mp, col_key, col_key_len), get_apr_error(msr->mp, rc));
+		msr->msc_sdbm_delete_error = 1;
             goto cleanup;
         }
 
 
         if (existing_dbm == NULL) {
             apr_sdbm_close(dbm);
+#ifdef GLOBAL_COLLECTION_LOCK
+            apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
             dbm = NULL;
         }
 
@@ -289,6 +321,9 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
             log_escape(msr->mp, col_name), log_escape_ex(msr->mp, col_key, col_key_len));
 
         apr_sdbm_close(dbm);
+#ifdef GLOBAL_COLLECTION_LOCK
+        apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
     }
 
     return col;
@@ -297,6 +332,9 @@ cleanup:
 
     if ((existing_dbm == NULL) && dbm) {
         apr_sdbm_close(dbm);
+#ifdef GLOBAL_COLLECTION_LOCK
+        apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
     }
 
     return NULL;
@@ -361,6 +399,16 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
                 log_escape(msr->mp, dbm_filename));
     }
 
+#ifdef GLOBAL_COLLECTION_LOCK
+    /* Need to lock to pull in the stored data again and apply deltas. */
+    rc = apr_global_mutex_lock(msr->modsecurity->dbm_lock);
+    if (rc != APR_SUCCESS) {
+        msr_log(msr, 1, "collection_store: Failed to lock proc mutex: %s",
+                get_apr_error(msr->mp, rc));
+        goto error;
+    }
+#endif
+
     /* Delete IS_NEW on store. */
     apr_table_unset(col, "IS_NEW");
 
@@ -417,12 +465,16 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
     rc = apr_sdbm_open(&dbm, dbm_filename, APR_CREATE | APR_WRITE | APR_SHARELOCK,
         CREATEMODE, msr->mp);
     if (rc != APR_SUCCESS) {
+#ifdef GLOBAL_COLLECTION_LOCK
+        apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
         msr_log(msr, 1, "collection_store: Failed to access DBM file \"%s\": %s", log_escape(msr->mp, dbm_filename),
             get_apr_error(msr->mp, rc));
         dbm = NULL;
         goto error;
     }
 
+#ifndef GLOBAL_COLLECTION_LOCK
     /* Need to lock to pull in the stored data again and apply deltas. */
     rc = apr_sdbm_lock(dbm, APR_FLOCK_EXCLUSIVE);
     if (rc != APR_SUCCESS) {
@@ -430,6 +482,7 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
             get_apr_error(msr->mp, rc));
         goto error;
     }
+#endif
 
     /* If there is an original value, then create a delta and
      * apply the delta to the current value */
@@ -479,10 +532,12 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
             }
         }
 
+        // Allocate blob_size for keys
         len = var->name_len + 1;
         if (len >= 65536) len = 65536;
         blob_size += len + 2;
 
+        // Allocate blob_size for values
         len = var->value_len + 1;
         if (len >= 65536) len = 65536;
         blob_size += len + 2;
@@ -492,8 +547,13 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
     blob = apr_pcalloc(msr->mp, blob_size);
     if (blob == NULL) {
         if (dbm != NULL) {
+#ifdef GLOBAL_COLLECTION_LOCK
+            apr_sdbm_close(dbm);
+            apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#else
             apr_sdbm_unlock(dbm);
             apr_sdbm_close(dbm);
+#endif
         }
 
         return -1;
@@ -550,15 +610,25 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
         msr_log(msr, 1, "collection_store: Failed to write to DBM file \"%s\": %s", dbm_filename,
                 get_apr_error(msr->mp, rc));
         if (dbm != NULL) {
+#ifdef GLOBAL_COLLECTION_LOCK
+            apr_sdbm_close(dbm);
+            apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#else
             apr_sdbm_unlock(dbm);
             apr_sdbm_close(dbm);
+#endif
         }
 
         return -1;
     }
 
+#ifdef GLOBAL_COLLECTION_LOCK
+    apr_sdbm_close(dbm);
+    apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#else
     apr_sdbm_unlock(dbm);
     apr_sdbm_close(dbm);
+#endif
 
     if (msr->txcfg->debuglog_level >= 4) {
         msr_log(msr, 4, "collection_store: Persisted collection (name \"%s\", key \"%s\").",
@@ -603,9 +673,21 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
                 log_escape(msr->mp, dbm_filename));
     }
 
+#ifdef GLOBAL_COLLECTION_LOCK
+    rc = apr_global_mutex_lock(msr->modsecurity->dbm_lock);
+    if (rc != APR_SUCCESS) {
+        msr_log(msr, 1, "collections_remove_stale: Failed to lock proc mutex: %s",
+                get_apr_error(msr->mp, rc));
+        goto error;
+    }
+#endif
+
     rc = apr_sdbm_open(&dbm, dbm_filename, APR_CREATE | APR_WRITE | APR_SHARELOCK,
             CREATEMODE, msr->mp);
     if (rc != APR_SUCCESS) {
+#ifdef GLOBAL_COLLECTION_LOCK
+        apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
         msr_log(msr, 1, "collections_remove_stale: Failed to access DBM file \"%s\": %s", log_escape(msr->mp, dbm_filename),
                 get_apr_error(msr->mp, rc));
         dbm = NULL;
@@ -614,12 +696,15 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
 
     /* First get a list of all keys. */
     keys_arr = apr_array_make(msr->mp, 256, sizeof(char *));
+
+#ifndef GLOBAL_COLLECTION_LOCK
     rc = apr_sdbm_lock(dbm, APR_FLOCK_SHARED);
     if (rc != APR_SUCCESS) {
         msr_log(msr, 1, "collections_remove_stale: Failed to lock DBM file \"%s\": %s", log_escape(msr->mp, dbm_filename),
             get_apr_error(msr->mp, rc));
         goto error;
     }
+#endif
 
     /* No one can write to the file while doing this so
      * do it as fast as possible.
@@ -632,7 +717,9 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
         }
         rc = apr_sdbm_nextkey(dbm, &key);
     }
+#ifndef GLOBAL_COLLECTION_LOCK
     apr_sdbm_unlock(dbm);
+#endif
 
     if (msr->txcfg->debuglog_level >= 9) {
         msr_log(msr, 9, "collections_remove_stale: Found %d record(s) in file \"%s\".", keys_arr->nelts,
@@ -678,10 +765,13 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
                 if (expiry_time <= now) {
                     rc = apr_sdbm_delete(dbm, key);
                     if (rc != APR_SUCCESS) {
-                        msr_log(msr, 1, "collections_remove_stale: Failed deleting collection (name \"%s\", "
+#ifdef LOG_NO_COLL_DELET_PB
+			if (msr->txcfg->debuglog_level >= 9)
+#endif
+			msr_log(msr, 1, "collections_remove_stale: Failed deleting collection (name \"%s\", "
                             "key \"%s\"): %s", log_escape(msr->mp, col_name),
                             log_escape_ex(msr->mp, key.dptr, key.dsize - 1), get_apr_error(msr->mp, rc));
-                    msr->msc_sdbm_delete_error = 1;
+			msr->msc_sdbm_delete_error = 1;
                         goto error;
                     }
 
@@ -698,13 +788,18 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
     }
 
     apr_sdbm_close(dbm);
-
+#ifdef GLOBAL_COLLECTION_LOCK
+    apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
     return 1;
 
 error:
 
     if (dbm) {
         apr_sdbm_close(dbm);
+#ifdef GLOBAL_COLLECTION_LOCK
+        apr_global_mutex_unlock(msr->modsecurity->dbm_lock);
+#endif
     }
 
     return -1;
